@@ -1,19 +1,21 @@
+from collections.abc import Generator
 from contextlib import contextmanager
-from typing import Generator, List, Optional
+from typing import Optional
 
 __all__ = ["CodeLines"]
 
 
 class CodeLines:
     def __init__(self) -> None:
-        self._lines: List[str] = []
+        self._lines: list[str] = []
         self._current_indent: str = ""
 
     def append(self, line: str) -> None:
         self._lines.append(f"{self._current_indent}{line}")
 
     def extend(self, lines: "CodeLines") -> None:
-        self._lines.extend(lines._lines)
+        for line in lines._lines:
+            self._lines.append(f"{self._current_indent}{line}")
 
     @contextmanager
     def indent(
@@ -34,8 +36,3 @@ class CodeLines:
     def reset(self) -> None:
         self._lines = []
         self._current_indent = ""
-
-    def branch_off(self) -> "CodeLines":
-        branch = CodeLines()
-        branch._current_indent = self._current_indent
-        return branch
