@@ -2,28 +2,22 @@ import datetime
 import enum
 import ipaddress
 import os
+import re
 import typing
 import uuid
+import zoneinfo
 from base64 import encodebytes
+from collections import ChainMap, Counter, OrderedDict, deque
+from collections.abc import Callable, Collection, Mapping, Sequence, Set
 from contextlib import suppress
 from dataclasses import is_dataclass
 from decimal import Decimal
 from fractions import Fraction
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    ForwardRef,
-    List,
-    Optional,
-    Tuple,
-    Type,
-    Union,
-)
+from typing import Any, ForwardRef, Optional, Tuple, Union
 
 import typing_extensions
 
-from mashumaro.core.const import PY_39_MIN, PY_311_MIN
+from mashumaro.core.const import PY_311_MIN
 from mashumaro.core.meta.code.lines import CodeLines
 from mashumaro.core.meta.helpers import (
     get_args,
@@ -80,10 +74,6 @@ from mashumaro.types import (
     SerializationStrategy,
 )
 
-if PY_39_MIN:
-    import zoneinfo
-
-
 __all__ = ["PackerRegistry"]
 
 
@@ -97,7 +87,7 @@ def _pack_with_annotated_serialization_strategy(
 ) -> Expression:
     strategy_type = type(strategy)
     try:
-        value_type: Union[Type, Any] = get_function_return_annotation(
+        value_type: Union[type, Any] = get_function_return_annotation(
             strategy.serialize
         )
     except (KeyError, ValueError):
@@ -251,7 +241,7 @@ def pack_dataclass(spec: ValueSpec) -> Optional[Expression]:
         if get_class_that_defines_method(
             method_name, method_loc
         ) != method_loc and (
-            spec.origin_type != spec.builder.cls
+            spec.origin_type is not spec.builder.cls
             or spec.builder.get_pack_method_name(
                 type_args=type_args,
                 format_name=spec.builder.format_name,
@@ -297,24 +287,40 @@ def pack_any(spec: ValueSpec) -> Optional[Expression]:
 
 
 def pack_union(
-    spec: ValueSpec, args: Tuple[Type, ...], prefix: str = "union"
+    spec: ValueSpec, args: tuple[type, ...], prefix: str = "union"
 ) -> Expression:
+    if spec.type is spec.owner and spec.field_ctx.packer:
+        return spec.field_ctx.packer
     lines = CodeLines()
+
     method_name = (
         f"__pack_{prefix}_{spec.builder.cls.__name__}_{spec.field_ctx.name}__"
         f"{random_hex()}"
     )
+
+    if not spec.field_ctx.packer:
+        method_args = ", ".join(
+            filter(None, ("value", spec.builder.get_pack_method_flags()))
+        )
+        if spec.builder.is_nailed:
+            union_packer = (
+                f"{spec.self_attrs_name}.{method_name}({method_args})"
+            )
+        else:
+            union_packer = f"{method_name}({method_args})"
+        spec.field_ctx.packer = union_packer
+
     method_args = "self, value" if spec.builder.is_nailed else "value"
     default_kwargs = spec.builder.get_pack_method_default_flag_values()
     if default_kwargs:
         lines.append(f"def {method_name}({method_args}, {default_kwargs}):")
     else:
         lines.append(f"def {method_name}({method_args}):")
-    packers: List[str] = []
-    packer_arg_types: Dict[str, List[Type]] = {}
+    packers: list[str] = []
+    packer_arg_types: dict[str, list[type]] = {}
     for type_arg in args:
         packer = PackerRegistry.get(
-            spec.copy(type=type_arg, expression="value")
+            spec.copy(type=type_arg, expression="value", owner=spec.type)
         )
         if packer not in packers:
             if packer == "value":
@@ -373,7 +379,9 @@ def pack_union(
     if spec.builder.get_config().debug:
         print(f"{type_name(spec.builder.cls)}:")
         print(lines.as_text())
+
     exec(lines.as_text(), spec.builder.globals, spec.builder.__dict__)
+
     method_args = ", ".join(
         filter(None, (spec.expression, spec.builder.get_pack_method_flags()))
     )
@@ -528,7 +536,7 @@ def pack_special_typing_primitive(spec: ValueSpec) -> Optional[Expression]:
             packer = PackerRegistry.get(spec.copy(type=get_args(spec.type)[0]))
             return f"*{packer}"
         elif is_type_var_tuple(spec.type):
-            return PackerRegistry.get(spec.copy(type=Tuple[Any, ...]))
+            return PackerRegistry.get(spec.copy(type=tuple[Any, ...]))
         elif isinstance(spec.type, ForwardRef):
             evaluated = spec.builder.evaluate_forward_ref(
                 spec.type, spec.owner
@@ -568,7 +576,7 @@ def pack_timezone(spec: ValueSpec) -> Optional[Expression]:
 
 @register
 def pack_zone_info(spec: ValueSpec) -> Optional[Expression]:
-    if PY_39_MIN and spec.origin_type is zoneinfo.ZoneInfo:
+    if spec.origin_type is zoneinfo.ZoneInfo:
         return f"str({spec.expression})"
 
 
@@ -603,10 +611,10 @@ def pack_fraction(spec: ValueSpec) -> Optional[Expression]:
         return f"str({spec.expression})"
 
 
-def pack_tuple(spec: ValueSpec, args: Tuple[Type, ...]) -> Expression:
+def pack_tuple(spec: ValueSpec, args: tuple[type, ...]) -> Expression:
     if not args:
         if spec.type in (Tuple, tuple):
-            args = [typing.Any, ...]  # type: ignore
+            args = [Any, ...]  # type: ignore
         else:
             return "[]"
     elif len(args) == 1 and args[0] == ():
@@ -618,7 +626,7 @@ def pack_tuple(spec: ValueSpec, args: Tuple[Type, ...]) -> Expression:
         )
         return f"[{packer} for value in {spec.expression}]"
     else:
-        arg_indexes: List[Union[int, Tuple[int, Union[int, None]]]] = []
+        arg_indexes: list[Union[int, tuple[int, Union[int, None]]]] = []
         unpack_idx: Optional[int] = None
         for arg_idx, type_arg in enumerate(args):
             if is_unpack(type_arg):
@@ -639,7 +647,7 @@ def pack_tuple(spec: ValueSpec, args: Tuple[Type, ...]) -> Expression:
                     arg_indexes.append(arg_idx)
                 else:
                     arg_indexes.append(arg_idx - len(args))
-        packers: List[Expression] = []
+        packers: list[Expression] = []
         for _idx, _arg_idx in enumerate(arg_indexes):
             if isinstance(_arg_idx, tuple):
                 p_expr = f"{spec.expression}[{_arg_idx[0]}:{_arg_idx[1]}]"
@@ -686,7 +694,7 @@ def pack_named_tuple(spec: ValueSpec) -> Expression:
     for idx, field in enumerate(fields):
         packer = PackerRegistry.get(
             spec.copy(
-                type=annotations.get(field, typing.Any),
+                type=annotations.get(field, Any),
                 expression=f"{spec.expression}[{idx}]",
                 could_be_none=True,
             )
@@ -761,7 +769,7 @@ def pack_typed_dict(spec: ValueSpec) -> Expression:
 
 @register
 def pack_collection(spec: ValueSpec) -> Optional[Expression]:
-    if not issubclass(spec.origin_type, typing.Collection):
+    if not issubclass(spec.origin_type, Collection):
         return None
     elif issubclass(spec.origin_type, enum.Enum):
         return None
@@ -769,7 +777,7 @@ def pack_collection(spec: ValueSpec) -> Optional[Expression]:
     args = get_args(spec.type)
 
     def inner_expr(
-        arg_num: int = 0, v_name: str = "value", v_type: Optional[Type] = None
+        arg_num: int = 0, v_name: str = "value", v_type: Optional[type] = None
     ) -> Expression:
         if v_type:
             return PackerRegistry.get(
@@ -810,38 +818,36 @@ def pack_collection(spec: ValueSpec) -> Optional[Expression]:
         return f"encodebytes({spec.expression}).decode()"
     elif issubclass(spec.origin_type, str):
         return spec.expression
-    elif issubclass(spec.origin_type, Tuple):  # type: ignore
+    elif issubclass(spec.origin_type, tuple):
         if is_named_tuple(spec.origin_type):
             return pack_named_tuple(spec)
         elif ensure_generic_collection(spec):
             return pack_tuple(spec, args)
-    elif ensure_generic_collection_subclass(
-        spec, typing.List, typing.Deque, typing.AbstractSet
-    ):
+    elif ensure_generic_collection_subclass(spec, list, deque, Set):
         ie = inner_expr()
         return _make_sequence_expression(ie)
-    elif ensure_generic_mapping(spec, args, typing.ChainMap):
+    elif ensure_generic_mapping(spec, args, ChainMap):
         ke = inner_expr(0, "key")
         ve = inner_expr(1)
         return (
             f"[{{{ke}: {ve} for key, value in m.items()}} "
             f"for m in {spec.expression}.maps]"
         )
-    elif ensure_generic_mapping(spec, args, typing.OrderedDict):
+    elif ensure_generic_mapping(spec, args, OrderedDict):
         ke = inner_expr(0, "key")
         ve = inner_expr(1)
         return _make_mapping_expression(ke, ve)
-    elif ensure_generic_mapping(spec, args, typing.Counter):
+    elif ensure_generic_mapping(spec, args, Counter):
         ke = inner_expr(0, "key")
         ve = inner_expr(1, v_type=int)
         return _make_mapping_expression(ke, ve)
     elif is_typed_dict(spec.origin_type):
         return pack_typed_dict(spec)
-    elif ensure_generic_mapping(spec, args, typing.Mapping):
+    elif ensure_generic_mapping(spec, args, Mapping):
         ke = inner_expr(0, "key")
         ve = inner_expr(1)
         return _make_mapping_expression(ke, ve)
-    elif ensure_generic_collection_subclass(spec, typing.Sequence):
+    elif ensure_generic_collection_subclass(spec, Sequence):
         ie = inner_expr()
         return _make_sequence_expression(ie)
 
@@ -856,3 +862,9 @@ def pack_pathlike(spec: ValueSpec) -> Optional[Expression]:
 def pack_enum(spec: ValueSpec) -> Optional[Expression]:
     if issubclass(spec.origin_type, enum.Enum):
         return f"{spec.expression}.value"
+
+
+@register
+def pack_pattern(spec: ValueSpec) -> Optional[Expression]:
+    if spec.origin_type in (typing.Pattern, re.Pattern):
+        return f"{spec.expression}.pattern"
