@@ -5,30 +5,30 @@ import enum
 import ipaddress
 import os
 import pathlib
+import re
 import types
 import typing
 import uuid
+import zoneinfo
 from abc import ABC
 from base64 import decodebytes
+from collections.abc import (
+    Callable,
+    Collection,
+    Iterable,
+    Mapping,
+    Sequence,
+    Set,
+)
 from contextlib import suppress
 from dataclasses import is_dataclass
 from decimal import Decimal
 from fractions import Fraction
-from typing import (
-    Any,
-    Callable,
-    ForwardRef,
-    Iterable,
-    List,
-    Optional,
-    Tuple,
-    Type,
-    Union,
-)
+from typing import Any, ForwardRef, Optional, Tuple, Union
 
 import typing_extensions
 
-from mashumaro.core.const import PY_39_MIN, PY_311_MIN
+from mashumaro.core.const import PY_311_MIN
 from mashumaro.core.helpers import parse_timezone
 from mashumaro.core.meta.code.lines import CodeLines
 from mashumaro.core.meta.helpers import (
@@ -68,6 +68,7 @@ from mashumaro.core.meta.types.common import (
     ExpressionWrapper,
     NoneType,
     Registry,
+    TypeMatchEligibleExpression,
     ValueSpec,
     clean_id,
     ensure_generic_collection,
@@ -90,9 +91,6 @@ from mashumaro.types import (
     SerializationStrategy,
 )
 
-if PY_39_MIN:
-    import zoneinfo
-
 try:
     import ciso8601
 except ImportError:  # pragma: no cover
@@ -132,7 +130,7 @@ class AbstractUnpackerBuilder(AbstractMethodBuilder, ABC):
         lines.append(f"def {method_name}({method_args}):")
         return method_name
 
-    def _get_extra_method_args(self) -> List[str]:
+    def _get_extra_method_args(self) -> list[str]:
         return []
 
     def _generate_method_args(self, spec: ValueSpec) -> str:
@@ -162,31 +160,69 @@ class AbstractUnpackerBuilder(AbstractMethodBuilder, ABC):
 
 
 class UnionUnpackerBuilder(AbstractUnpackerBuilder):
-    def __init__(self, args: Tuple[Type, ...]):
+    def __init__(self, args: tuple[type, ...]):
         self.union_args = args
+        self.method_name: Optional[str] = None
 
     def get_method_prefix(self) -> str:
         return "union"
 
+    def _generate_method_name(self, spec: ValueSpec) -> str:
+        method_name = super()._generate_method_name(spec)
+        self.method_name = method_name
+        return method_name
+
     def _add_body(self, spec: ValueSpec, lines: CodeLines) -> None:
-        ambiguous_unpacker_types = []
+        if not spec.field_ctx.unpacker and self.method_name:
+            spec.field_ctx.unpacker = self._get_call_expr(
+                spec, self.method_name
+            )
+        orig_lines = lines
+        lines = CodeLines()
+        unpackers = set()
+        fallback_unpackers = []
+        type_arg_unpackers = []
+        type_match_statements = 0
         for type_arg in self.union_args:
             unpacker = UnpackerRegistry.get(
-                spec.copy(type=type_arg, expression="value")
+                spec.copy(type=type_arg, expression="value", owner=spec.type)
             )
-            if type_arg in (bool, str) and unpacker == "value":
-                ambiguous_unpacker_types.append(type_arg)
+            type_arg_unpackers.append((type_arg, unpacker))
+            if isinstance(unpacker, TypeMatchEligibleExpression):
+                type_match_statements += 1
+        for type_arg, unpacker in type_arg_unpackers:
+            condition = ""
+            do_try = unpacker != "value"
+            unpacker_block = CodeLines()
+            if isinstance(unpacker, TypeMatchEligibleExpression):
+                do_try = False
+                if type_match_statements > 1:
+                    condition = f"__value_type is {type_arg.__name__}"
+                else:
+                    condition = f"type(value) is {type_arg.__name__}"
+                if (condition, unpacker) in unpackers:  # pragma: no cover
+                    # we shouldn't be here because condition is always unique
+                    continue
+                with unpacker_block.indent(f"if {condition}:"):
+                    unpacker_block.append("return value")
+                if (condition, unpacker) not in unpackers:
+                    fallback_unpackers.append(unpacker)
+            elif (condition, unpacker) in unpackers:
+                continue
+            else:
+                unpacker_block.append(f"return {unpacker}")
+
+            if do_try:
+                with lines.indent("try:"):
+                    lines.extend(unpacker_block)
+                lines.append("except Exception: pass")
+            else:
+                lines.extend(unpacker_block)
+            unpackers.add((condition, unpacker))
+        for fallback_unpacker in fallback_unpackers:
             with lines.indent("try:"):
-                lines.append(f"return {unpacker}")
+                lines.append(f"return {fallback_unpacker}")
             lines.append("except Exception: pass")
-        # if len(ambiguous_unpacker_types) >= 2:
-        #     warnings.warn(
-        #         f"{type_name(spec.builder.cls)}.{spec.field_ctx.name} "
-        #         f"({type_name(spec.type)}): "
-        #         "In the next release, data marked with Union type "
-        #         "containing 'str' and 'bool' will be coerced to the value "
-        #         "of the type specified first instead of passing it as is"
-        #     )
         field_type = spec.builder.get_type_name_identifier(
             typ=spec.type,
             resolved_type_params=spec.builder.get_field_resolved_type_params(
@@ -200,6 +236,13 @@ class UnionUnpackerBuilder(AbstractUnpackerBuilder):
             )
         else:
             lines.append("raise ValueError(value)")
+        if type_match_statements > 1:
+            orig_lines.append("__value_type = type(value)")
+        orig_lines.extend(lines)
+
+    def _get_existing_method(self, spec: ValueSpec) -> Optional[str]:
+        if spec.owner is spec.type:
+            return spec.field_ctx.unpacker
 
 
 class TypeVarUnpackerBuilder(UnionUnpackerBuilder):
@@ -248,7 +291,7 @@ class DiscriminatedUnionUnpackerBuilder(AbstractUnpackerBuilder):
     def __init__(
         self,
         discriminator: Discriminator,
-        base_variants: Optional[Tuple[Type, ...]] = None,
+        base_variants: Optional[tuple[type, ...]] = None,
     ):
         self.discriminator = discriminator
         self.base_variants = base_variants or tuple()
@@ -257,7 +300,7 @@ class DiscriminatedUnionUnpackerBuilder(AbstractUnpackerBuilder):
     def get_method_prefix(self) -> str:
         return ""
 
-    def _get_extra_method_args(self) -> List[str]:
+    def _get_extra_method_args(self) -> list[str]:
         return ["_dialect", "_default_dialect"]
 
     def _get_variants_attr(self, spec: ValueSpec) -> str:
@@ -275,9 +318,9 @@ class DiscriminatedUnionUnpackerBuilder(AbstractUnpackerBuilder):
         else:
             return f"{spec.cls_attrs_name}.{variants_attr}"
 
-    def _get_variant_names(self, spec: ValueSpec) -> List[str]:
+    def _get_variant_names(self, spec: ValueSpec) -> list[str]:
         base_variants = self.base_variants or (spec.origin_type,)
-        variant_names: List[str] = []
+        variant_names: list[str] = []
         if self.discriminator.include_subtypes:
             spec.builder.ensure_object_imported(iter_all_subclasses)
             variant_names.extend(
@@ -301,7 +344,7 @@ class DiscriminatedUnionUnpackerBuilder(AbstractUnpackerBuilder):
         return f'({", ".join(variant_names)})'
 
     @staticmethod
-    def _get_variants_attr_holder(spec: ValueSpec) -> Type:
+    def _get_variants_attr_holder(spec: ValueSpec) -> type:
         return spec.attrs
 
     @staticmethod
@@ -511,7 +554,7 @@ def _unpack_with_annotated_serialization_strategy(
 ) -> Expression:
     strategy_type = type(strategy)
     try:
-        value_type: Union[Type, Any] = get_function_arg_annotation(
+        value_type: Union[type, Any] = get_function_arg_annotation(
             strategy.deserialize, arg_pos=0
         )
     except (KeyError, ValueError):
@@ -665,7 +708,7 @@ def unpack_dataclass(spec: ValueSpec) -> Optional[Expression]:
         if get_class_that_defines_method(
             method_name, method_loc
         ) != method_loc and (
-            spec.origin_type != spec.builder.cls
+            spec.origin_type is not spec.builder.cls
             or spec.builder.get_unpack_method_name(
                 type_args=type_args,
                 format_name=spec.builder.format_name,
@@ -821,7 +864,7 @@ def unpack_special_typing_primitive(spec: ValueSpec) -> Optional[Expression]:
             )
             return f"*{unpacker}"
         elif is_type_var_tuple(spec.type):
-            return UnpackerRegistry.get(spec.copy(type=Tuple[Any, ...]))
+            return UnpackerRegistry.get(spec.copy(type=tuple[Any, ...]))
         elif isinstance(spec.type, ForwardRef):
             evaluated = spec.builder.evaluate_forward_ref(
                 spec.type, spec.owner
@@ -838,13 +881,21 @@ def unpack_special_typing_primitive(spec: ValueSpec) -> Optional[Expression]:
 @register
 def unpack_number(spec: ValueSpec) -> Optional[Expression]:
     if spec.origin_type in (int, float):
-        return f"{type_name(spec.origin_type)}({spec.expression})"
+        return TypeMatchEligibleExpression(
+            f"{type_name(spec.origin_type)}({spec.expression})"
+        )
 
 
 @register
-def unpack_bool_and_none(spec: ValueSpec) -> Optional[Expression]:
-    if spec.origin_type in (bool, NoneType, None):
-        return spec.expression
+def unpack_bool(spec: ValueSpec) -> Optional[Expression]:
+    if spec.origin_type is bool:
+        return TypeMatchEligibleExpression(f"bool({spec.expression})")
+
+
+@register
+def unpack_none(spec: ValueSpec) -> Optional[Expression]:
+    if spec.origin_type in (NoneType, None):
+        return TypeMatchEligibleExpression("None")
 
 
 @register
@@ -906,7 +957,7 @@ def unpack_timezone(spec: ValueSpec) -> Optional[Expression]:
 
 @register
 def unpack_zone_info(spec: ValueSpec) -> Optional[Expression]:
-    if PY_39_MIN and spec.origin_type is zoneinfo.ZoneInfo:
+    if spec.origin_type is zoneinfo.ZoneInfo:
         method = "__zoneinfo_ZoneInfo"
         spec.builder.ensure_object_imported(zoneinfo.ZoneInfo, method)
         return f"{method}({spec.expression})"
@@ -949,10 +1000,10 @@ def unpack_fraction(spec: ValueSpec) -> Optional[Expression]:
         return f"Fraction({spec.expression})"
 
 
-def unpack_tuple(spec: ValueSpec, args: Tuple[Type, ...]) -> Expression:
+def unpack_tuple(spec: ValueSpec, args: tuple[type, ...]) -> Expression:
     if not args:
         if spec.type in (Tuple, tuple):
-            args = [typing.Any, ...]  # type: ignore
+            args = [Any, ...]  # type: ignore
         else:
             return "()"
     elif len(args) == 1 and args[0] == ():
@@ -964,7 +1015,7 @@ def unpack_tuple(spec: ValueSpec, args: Tuple[Type, ...]) -> Expression:
         )
         return f"tuple([{unpacker} for value in {spec.expression}])"
     else:
-        arg_indexes: List[Union[int, Tuple[int, Union[int, None]]]] = []
+        arg_indexes: list[Union[int, tuple[int, Union[int, None]]]] = []
         unpack_idx: Optional[int] = None
         for arg_idx, type_arg in enumerate(args):
             if is_unpack(type_arg):
@@ -985,7 +1036,7 @@ def unpack_tuple(spec: ValueSpec, args: Tuple[Type, ...]) -> Expression:
                     arg_indexes.append(arg_idx)
                 else:
                     arg_indexes.append(arg_idx - len(args))
-        unpackers: List[Expression] = []
+        unpackers: list[Expression] = []
         for _idx, _arg_idx in enumerate(arg_indexes):
             if isinstance(_arg_idx, tuple):
                 u_expr = f"{spec.expression}[{_arg_idx[0]}:{_arg_idx[1]}]"
@@ -1158,7 +1209,7 @@ def unpack_typed_dict(spec: ValueSpec) -> Expression:
 
 @register
 def unpack_collection(spec: ValueSpec) -> Optional[Expression]:
-    if not issubclass(spec.origin_type, typing.Collection):
+    if not issubclass(spec.origin_type, Collection):
         return None
     elif issubclass(spec.origin_type, enum.Enum):
         return None
@@ -1166,7 +1217,7 @@ def unpack_collection(spec: ValueSpec) -> Optional[Expression]:
     args = get_args(spec.type)
 
     def inner_expr(
-        arg_num: int = 0, v_name: str = "value", v_type: Optional[Type] = None
+        arg_num: int = 0, v_name: str = "value", v_type: Optional[type] = None
     ) -> Expression:
         if v_type:
             return UnpackerRegistry.get(
@@ -1194,37 +1245,37 @@ def unpack_collection(spec: ValueSpec) -> Optional[Expression]:
             spec.builder.ensure_object_imported(decodebytes)
             return f"bytearray(decodebytes({spec.expression}.encode()))"
     elif issubclass(spec.origin_type, str):
-        return spec.expression
-    elif ensure_generic_collection_subclass(spec, List):
+        return TypeMatchEligibleExpression(f"str({spec.expression})")
+    elif ensure_generic_collection_subclass(spec, list):
         return f"[{inner_expr()} for value in {spec.expression}]"
-    elif ensure_generic_collection_subclass(spec, typing.Deque):
+    elif ensure_generic_collection_subclass(spec, collections.deque):
         spec.builder.ensure_module_imported(collections)
         return (
             f"collections.deque([{inner_expr()} "
             f"for value in {spec.expression}])"
         )
-    elif issubclass(spec.origin_type, Tuple):  # type: ignore
+    elif issubclass(spec.origin_type, tuple):  # type: ignore
         if is_named_tuple(spec.origin_type):
             return unpack_named_tuple(spec)
         elif ensure_generic_collection(spec):
             return unpack_tuple(spec, args)
-    elif ensure_generic_collection_subclass(spec, typing.FrozenSet):
+    elif ensure_generic_collection_subclass(spec, frozenset):
         return f"frozenset([{inner_expr()} for value in {spec.expression}])"
-    elif ensure_generic_collection_subclass(spec, typing.AbstractSet):
+    elif ensure_generic_collection_subclass(spec, Set):
         return f"set([{inner_expr()} for value in {spec.expression}])"
-    elif ensure_generic_mapping(spec, args, typing.ChainMap):
+    elif ensure_generic_mapping(spec, args, collections.ChainMap):
         spec.builder.ensure_module_imported(collections)
         return (
             f'collections.ChainMap(*[{{{inner_expr(0, "key")}:{inner_expr(1)} '
             f"for key, value in m.items()}} for m in {spec.expression}])"
         )
-    elif ensure_generic_mapping(spec, args, typing.OrderedDict):
+    elif ensure_generic_mapping(spec, args, collections.OrderedDict):
         spec.builder.ensure_module_imported(collections)
         return (
             f'collections.OrderedDict({{{inner_expr(0, "key")}: '
             f"{inner_expr(1)} for key, value in {spec.expression}.items()}})"
         )
-    elif ensure_generic_mapping(spec, args, typing.DefaultDict):
+    elif ensure_generic_mapping(spec, args, collections.defaultdict):
         spec.builder.ensure_module_imported(collections)
         default_type = type_name(args[1] if args else None)
         return (
@@ -1232,7 +1283,7 @@ def unpack_collection(spec: ValueSpec) -> Optional[Expression]:
             f"{{{inner_expr(0, 'key')}: "
             f"{inner_expr(1)} for key, value in {spec.expression}.items()}})"
         )
-    elif ensure_generic_mapping(spec, args, typing.Counter):
+    elif ensure_generic_mapping(spec, args, collections.Counter):
         spec.builder.ensure_module_imported(collections)
         return (
             f'collections.Counter({{{inner_expr(0, "key")}: '
@@ -1247,12 +1298,12 @@ def unpack_collection(spec: ValueSpec) -> Optional[Expression]:
             f'types.MappingProxyType({{{inner_expr(0, "key")}: {inner_expr(1)}'
             f" for key, value in {spec.expression}.items()}})"
         )
-    elif ensure_generic_mapping(spec, args, typing.Mapping):
+    elif ensure_generic_mapping(spec, args, Mapping):
         return (
             f'{{{inner_expr(0, "key")}: {inner_expr(1)} '
             f"for key, value in {spec.expression}.items()}}"
         )
-    elif ensure_generic_collection_subclass(spec, typing.Sequence):
+    elif ensure_generic_collection_subclass(spec, Sequence):
         return f"[{inner_expr()} for value in {spec.expression}]"
 
 
@@ -1271,3 +1322,11 @@ def unpack_enum(spec: ValueSpec) -> Optional[Expression]:
     if issubclass(spec.origin_type, enum.Enum):
         field_type = spec.builder.get_type_name_identifier(spec.origin_type)
         return f"{field_type}({spec.expression})"
+
+
+@register
+def unpack_pattern(spec: ValueSpec) -> Optional[Expression]:
+    if spec.origin_type in (typing.Pattern, re.Pattern):
+        method = "__re_compile"
+        spec.builder.ensure_object_imported(re.compile, method)
+        return f"{method}({spec.expression})"
