@@ -4,7 +4,6 @@ from typing import ForwardRef, TypedDict
 
 import pytest
 
-from mashumaro.core.const import PY_39_MIN
 from mashumaro.core.meta.helpers import get_function_arg_annotation
 from mashumaro.jsonschema import build_json_schema
 from mashumaro.jsonschema.models import (
@@ -18,13 +17,6 @@ class MyTypedDict(TypedDict):
     x: int
 
 
-@pytest.mark.skipif(
-    not PY_39_MIN,
-    reason=(
-        "On Python 3.8 ForwardRef doesn't have __forward_module__ "
-        "which is needed here"
-    ),
-)
 def test_jsonschema_generation_for_forward_refs():
     def foo(x: int, y: MyTypedDict): ...
 
